@@ -30,6 +30,8 @@
 #include <QVBoxLayout>
 #include <QAudioBufferOutput>
 
+using namespace Qt::Literals;
+
 Player::Player(QWidget *parent) : QWidget(parent)
 {
     //! [create-objs]
@@ -134,6 +136,36 @@ Player::Player(QWidget *parent) : QWidget(parent)
     m_fullScreenButton->setCheckable(true);
     controlLayout->addWidget(m_fullScreenButton);
 
+    m_pitchCompensationButton = new QPushButton(tr("Pitch compensation"), this);
+    m_pitchCompensationButton->setCheckable(true);
+    switch (m_player->pitchCompensationAvailability()) {
+    case QMediaPlayer::PitchCompensationAvailability::AlwaysOn:
+        m_pitchCompensationButton->setEnabled(false);
+        m_pitchCompensationButton->setChecked(true);
+        m_pitchCompensationButton->setToolTip(
+                u"Pitch compensation always enabled on this backend"_s);
+        break;
+    case QMediaPlayer::PitchCompensationAvailability::Unavailable:
+        m_pitchCompensationButton->setEnabled(false);
+        m_pitchCompensationButton->setChecked(false);
+        m_pitchCompensationButton->setToolTip(u"Pitch compensation unavailable on this backend"_s);
+        break;
+    case QMediaPlayer::PitchCompensationAvailability::Available:
+        m_pitchCompensationButton->setEnabled(true);
+        m_pitchCompensationButton->setChecked(m_player->pitchCompensation());
+        break;
+    default:
+        Q_UNREACHABLE();
+    }
+
+    controlLayout->addWidget(m_pitchCompensationButton);
+    connect(m_player, &QMediaPlayer::pitchCompensationChanged, this, [this] {
+        m_pitchCompensationButton->setChecked(m_player->pitchCompensation());
+    });
+    m_pitchCompensationButton->setChecked(m_player->pitchCompensation());
+    connect(m_pitchCompensationButton, &QPushButton::toggled, m_player,
+            &QMediaPlayer::setPitchCompensation);
+
 #if !defined(Q_OS_ANDROID) && !defined(Q_OS_IOS)
     m_audioOutputCombo = new QComboBox(this);
     controlLayout->addWidget(m_audioOutputCombo);
@@ -289,8 +321,8 @@ void Player::metaDataChanged()
 {
     auto metaData = m_player->metaData();
     setTrackInfo(QStringLiteral("%1 - %2")
-                         .arg(metaData.value(QMediaMetaData::AlbumArtist).toString())
-                         .arg(metaData.value(QMediaMetaData::Title).toString()));
+                         .arg(metaData.value(QMediaMetaData::AlbumArtist).toString(),
+                              metaData.value(QMediaMetaData::Title).toString()));
 
 #if !defined(Q_OS_ANDROID) && !defined(Q_OS_IOS)
     for (int i = 0; i < QMediaMetaData::NumMetaData; i++) {
@@ -326,7 +358,9 @@ void Player::metaDataChanged()
     const int currentVideoTrack = m_player->activeVideoTrack();
     if (currentVideoTrack >= 0 && currentVideoTrack < tracks.size()) {
         const QMediaMetaData track = tracks.value(currentVideoTrack);
-        for (const QMediaMetaData::Key &key : track.keys()) {
+
+        const QList<QMediaMetaData::Key> trackKeys = track.keys();
+        for (const QMediaMetaData::Key &key : std::as_const(trackKeys)) {
             if (QLineEdit *field = qobject_cast<QLineEdit *>(m_metaDataFields[key])) {
                 QString stringValue = track.stringValue(key);
                 field->setText(stringValue);
@@ -353,7 +387,7 @@ QString Player::trackName(const QMediaMetaData &metaData, int index)
         if (lang == QLocale::Language::AnyLanguage)
             name = title;
         else
-            name = QStringLiteral("%1 - [%2]").arg(title).arg(QLocale::languageToString(lang));
+            name = QStringLiteral("%1 - [%2]").arg(title, QLocale::languageToString(lang));
     }
     return name;
 }
@@ -508,7 +542,7 @@ void Player::setTrackInfo(const QString &info)
         m_statusLabel->setText(m_statusInfo);
     } else {
         if (!m_statusInfo.isEmpty())
-            setWindowTitle(QStringLiteral("%1 | %2").arg(m_trackInfo).arg(m_statusInfo));
+            setWindowTitle(QStringLiteral("%1 | %2").arg(m_trackInfo, m_statusInfo));
         else
             setWindowTitle(m_trackInfo);
     }
@@ -523,7 +557,7 @@ void Player::setStatusInfo(const QString &info)
         m_statusLabel->setText(m_statusInfo);
     } else {
         if (!m_statusInfo.isEmpty())
-            setWindowTitle(QStringLiteral("%1 | %2").arg(m_trackInfo).arg(m_statusInfo));
+            setWindowTitle(QStringLiteral("%1 | %2").arg(m_trackInfo, m_statusInfo));
         else
             setWindowTitle(m_trackInfo);
     }
