@@ -3,7 +3,7 @@
 
 #include "qaudiosystem_p.h"
 #include "qaudiodevice_p.h"
-#include <private/qplatformmediadevices_p.h>
+#include <private/qplatformaudiodevices_p.h>
 #include <private/qplatformmediaintegration_p.h>
 
 #include <QtCore/qmap.h>
@@ -30,6 +30,11 @@ QT_DEFINE_QESDP_SPECIALIZATION_DTOR(QAudioDevicePrivate);
     QAudioSource, and QAudioSink. It is also used to determine the
     input or output device to use in a capture session or during media playback.
 
+    The QAudioDevice instance retains its properties throughout its lifetime,
+    even if the corresponding physical device is disconnected or its settings are
+    modified. To keep track of updated properties, the user should load new instances
+    of QAudioDevice from \l{QMediaDevices} when the relevant signals are fired.
+
     You can also query each device for the formats it supports. A
     format in this context is a set consisting of a channel count, sample rate, and sample type. A
     format is represented by the QAudioFormat class.
@@ -60,7 +65,7 @@ QT_DEFINE_QESDP_SPECIALIZATION_DTOR(QAudioDevicePrivate);
     \qmlvaluetype audioDevice
     \inqmlmodule QtMultimedia
     \since 6.2
-    //! \instantiates QAudioDevice
+    //! \nativetype QAudioDevice
     \brief Describes an audio device.
     \ingroup multimedia_qml
     \ingroup multimedia_audio_qml
@@ -69,6 +74,11 @@ QT_DEFINE_QESDP_SPECIALIZATION_DTOR(QAudioDevicePrivate);
     The audioDevice value type describes the properties of an audio device that
     is connected to the system.
 
+    The audioDevice instance retains its properties throughout its lifetime,
+    even if the corresponding physical device is disconnected or its settings are
+    modified. To keep track of updated properties, the user should load new instances
+    of audioDevice from \l{MediaDevices} when the relevant signals are fired.
+
     The list of audio input or output devices can be queried from the \l{MediaDevices}
     type. To select a certain audio device for input or output set it as the device
     on \l{AudioInput} or \l{AudioOutput}.
@@ -127,13 +137,7 @@ QAudioDevice &QAudioDevice::operator=(const QAudioDevice &other) = default;
 */
 bool QAudioDevice::operator==(const QAudioDevice &other) const
 {
-    if (d == other.d)
-        return true;
-    if (!d || !other.d)
-        return false;
-    if (d->mode == other.d->mode && d->id == other.d->id && d->isDefault == other.d->isDefault)
-        return true;
-    return false;
+    return mode() == other.mode() && id() == other.id();
 }
 
 /*!
@@ -326,16 +330,11 @@ QAudioDevice::QAudioDevice(QAudioDevicePrivate *p) : d(p) { }
 
 /*!
     \qmlproperty enumeration QtMultimedia::audioDevice::mode
+    \qmlenumeratorsfrom [AudioDevice] QAudioDevice::Mode
 
-   Holds whether this device is an input or output device.
+    Holds whether this device is an input or output device.
 
     The returned value can be one of the following:
-
-
-    \value audioDevice.Null A null device.
-    \value audioDevice.Input input device.
-    \value audioDevice.Output An output device.
-
 */
 
 /*!
@@ -368,6 +367,12 @@ QDebug operator<<(QDebug dbg, QAudioDevice::Mode mode)
 }
 #endif
 
+QAudioDevice
+QAudioDevicePrivate::createQAudioDevice(std::unique_ptr<QAudioDevicePrivate> devicePrivate)
+{
+    return QAudioDevice(devicePrivate.release());
+}
+
 QT_END_NAMESPACE
 
 #include "moc_qaudiodevice.cpp"
