@@ -15,19 +15,17 @@
 // We mean it.
 //
 
-#include <QtCore/QMutex>
 #include "private/qplatformvideosink_p.h"
 
-Q_FORWARD_DECLARE_OBJC_CLASS(CALayer);
-Q_FORWARD_DECLARE_OBJC_CLASS(AVPlayerLayer);
-Q_FORWARD_DECLARE_OBJC_CLASS(AVCaptureVideoPreviewLayer);
+#include <QtCore/private/qcore_mac_p.h>
 
 #include <CoreVideo/CVBase.h>
-#include <CoreVideo/CVPixelBuffer.h>
 #include <CoreVideo/CVImageBuffer.h>
+#include <CoreVideo/CVPixelBuffer.h>
 
-#import "Metal/Metal.h"
-#import "MetalKit/MetalKit.h"
+#import <AvFoundation/AVFoundation.h>
+#import <Metal/Metal.h>
+#import <MetalKit/MetalKit.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -38,8 +36,8 @@ class AVFVideoSink : public QPlatformVideoSink
     Q_OBJECT
 
 public:
-    AVFVideoSink(QVideoSink *parent = nullptr);
-    virtual ~AVFVideoSink();
+    explicit AVFVideoSink(QVideoSink *parent = nullptr);
+    ~AVFVideoSink() override;
 
     // QPlatformVideoSink interface
 public:
@@ -65,9 +63,7 @@ public:
     virtual void reconfigure() = 0;
     virtual void setRhi(QRhi *);
     virtual void setLayer(CALayer *layer);
-    virtual void setOutputSettings(NSDictionary *settings);
-
-    QMutex *textureCacheMutex() { return &m_textureCacheMutex; }
+    virtual void setOutputSettings();
 
     QRhi *rhi() const { return m_rhi; }
 
@@ -75,11 +71,11 @@ public:
     void nativeSizeChanged() { updateLayerBounds(); }
     QSize nativeSize() const { return m_sink ? m_sink->nativeSize() : QSize(); }
 
-    CVMetalTextureCacheRef cvMetalTextureCache = nullptr;
+    QCFType<CVMetalTextureCacheRef> cvMetalTextureCache;
 #if defined(Q_OS_MACOS)
-    CVOpenGLTextureCacheRef cvOpenGLTextureCache = nullptr;
+    QCFType<CVOpenGLTextureCacheRef> cvOpenGLTextureCache;
 #elif defined(Q_OS_IOS)
-    CVOpenGLESTextureCacheRef cvOpenGLESTextureCache = nullptr;
+    QCFType<CVOpenGLESTextureCacheRef> cvOpenGLESTextureCache;
 #endif
 private:
     void freeTextureCaches();
@@ -90,7 +86,6 @@ protected:
     QRhi *m_rhi = nullptr;
     CALayer *m_layer = nullptr;
     NSDictionary *m_outputSettings = nullptr;
-    QMutex m_textureCacheMutex;
 };
 
 
