@@ -7,8 +7,8 @@
 #include "avfcamerasession_p.h"
 #include "avfcamera_p.h"
 #include "avfcameraservice_p.h"
-#include "avfcameradebug_p.h"
-#include "avfcamerautility_p.h"
+#include <QtMultimedia/private/qavfcameradebug_p.h>
+#include <QtMultimedia/private/qavfcamerautility_p.h>
 #include "qaudiodevice.h"
 
 #include "qmediadevices.h"
@@ -20,7 +20,6 @@
 
 #include <QtCore/qmath.h>
 #include <QtCore/qdebug.h>
-#include <QtCore/qmimetype.h>
 
 #include <private/qcoreaudioutils_p.h>
 
@@ -53,7 +52,7 @@ bool qt_file_exists(NSURL *fileURL)
     return false;
 }
 
-}
+} // namespace
 
 AVFMediaEncoder::AVFMediaEncoder(QMediaRecorder *parent)
     : QObject(parent)
@@ -209,8 +208,10 @@ static NSDictionary *avfAudioSettings(const QMediaEncoderSettings &encoderSettin
         // fallback to providing channel layout if channel count is not specified or supported
         UInt32 size = 0;
         if (format.isValid()) {
-            auto layout = CoreAudioUtils::toAudioChannelLayout(format, &size);
-            [settings setObject:[NSData dataWithBytes:layout.get() length:sizeof(AudioChannelLayout)] forKey:AVChannelLayoutKey];
+            auto layout = QCoreAudioUtils::toAudioChannelLayout(format, &size);
+            UInt32 layoutSize = offsetof(AudioChannelLayout, mChannelDescriptions)
+                    + layout->mNumberChannelDescriptions * sizeof(AudioChannelDescription);
+            [settings setObject:[NSData dataWithBytes:layout.get() length:layoutSize] forKey:AVChannelLayoutKey];
         } else {
             // finally default to setting channel count to 1
             [settings setObject:[NSNumber numberWithInt:1] forKey:AVNumberOfChannelsKey];
@@ -230,7 +231,9 @@ static NSDictionary *avfAudioSettings(const QMediaEncoderSettings &encoderSettin
     return settings;
 }
 
-NSDictionary *avfVideoSettings(QMediaEncoderSettings &encoderSettings, AVCaptureDevice *device, AVCaptureConnection *connection, QSize nativeSize)
+static NSDictionary *avfVideoSettings(QMediaEncoderSettings &encoderSettings,
+                                      AVCaptureDevice *device, AVCaptureConnection *connection,
+                                      QSize nativeSize)
 {
     if (!device)
         return nil;
@@ -479,10 +482,15 @@ void AVFMediaEncoder::record(QMediaEncoderSettings &settings)
 
     if (!cameraControl && !audioInput) {
         qWarning() << Q_FUNC_INFO << "Cannot record without any inputs";
-        Q_EMIT error(QMediaRecorder::ResourceError, tr("No inputs specified"));
+        updateError(QMediaRecorder::ResourceError, tr("No inputs specified"));
         return;
     }
 
+    // This is necessary to explicitly recreate m_audioInput inside AVFCameraSession.
+    // Which in turn is necessary for the case when the microphone was disconnected
+    // after stopping recording and reconnected.
+    m_service->session()->updateAudioInput();
+
     m_service->session()->setActive(true);
     const bool audioOnly = settings.videoCodec() == QMediaFormat::VideoCodec::Unspecified;
     AVCaptureSession *session = m_service->session()->captureSession();
@@ -491,28 +499,28 @@ void AVFMediaEncoder::record(QMediaEncoderSettings &settings)
     if (!audioOnly) {
         if (!cameraControl || !cameraControl->isActive()) {
             qCDebug(qLcCamera) << Q_FUNC_INFO << "can not start record while camera is not active";
-            Q_EMIT error(QMediaRecorder::ResourceError,
-                         QMediaRecorderPrivate::msgFailedStartRecording());
+            updateError(QMediaRecorder::ResourceError,
+                        QMediaRecorderPrivate::msgFailedStartRecording());
             return;
         }
     }
 
     const QString path(outputLocation().scheme() == QLatin1String("file") ?
                            outputLocation().path() : outputLocation().toString());
-    const QUrl fileURL(QUrl::fromLocalFile(QMediaStorageLocation::generateFileName(path,
-                    audioOnly ? QStandardPaths::MusicLocation : QStandardPaths::MoviesLocation,
-                    settings.mimeType().preferredSuffix())));
+    const QUrl fileURL(QUrl::fromLocalFile(QMediaStorageLocation::generateFileName(
+            path, audioOnly ? QStandardPaths::MusicLocation : QStandardPaths::MoviesLocation,
+            settings.preferredSuffix())));
 
     NSURL *nsFileURL = fileURL.toNSURL();
     if (!nsFileURL) {
         qWarning() << Q_FUNC_INFO << "invalid output URL:" << fileURL;
-        Q_EMIT error(QMediaRecorder::ResourceError, tr("Invalid output file URL"));
+        updateError(QMediaRecorder::ResourceError, tr("Invalid output file URL"));
         return;
     }
     if (!qt_is_writable_file_URL(nsFileURL)) {
         qWarning() << Q_FUNC_INFO << "invalid output URL:" << fileURL
                     << "(the location is not writable)";
-        Q_EMIT error(QMediaRecorder::ResourceError, tr("Non-writeable file location"));
+        updateError(QMediaRecorder::ResourceError, tr("Non-writeable file location"));
         return;
     }
     if (qt_file_exists(nsFileURL)) {
@@ -520,7 +528,7 @@ void AVFMediaEncoder::record(QMediaEncoderSettings &settings)
         // Objective-C exception, which is not good at all.
         qWarning() << Q_FUNC_INFO << "invalid output URL:" << fileURL
                     << "(file already exists)";
-        Q_EMIT error(QMediaRecorder::ResourceError, tr("File already exists"));
+        updateError(QMediaRecorder::ResourceError, tr("File already exists"));
         return;
     }
 
@@ -541,8 +549,8 @@ void AVFMediaEncoder::record(QMediaEncoderSettings &settings)
 
         m_state = QMediaRecorder::RecordingState;
 
-        Q_EMIT actualLocationChanged(fileURL);
-        Q_EMIT stateChanged(m_state);
+        actualLocationChanged(fileURL);
+        stateChanged(m_state);
 
         // Apple recommends to call startRunning and do all
         // setup on a special queue, and that's what we had
@@ -555,8 +563,7 @@ void AVFMediaEncoder::record(QMediaEncoderSettings &settings)
         [m_writer start];
     } else {
         [session startRunning];
-        Q_EMIT error(QMediaRecorder::FormatError,
-                     QMediaRecorderPrivate::msgFailedStartRecording());
+        updateError(QMediaRecorder::FormatError, QMediaRecorderPrivate::msgFailedStartRecording());
     }
 }
 
@@ -621,17 +628,18 @@ void AVFMediaEncoder::assetWriterFinished()
         if (session->audioPreviewDelegate()) {
             [session->audioPreviewDelegate() resetAudioPreviewDelegate];
         }
-        [session->captureSession() startRunning];
+        if (session->videoOutput() || session->audioPreviewDelegate())
+            [session->captureSession() startRunning];
     }
 
     m_state = QMediaRecorder::StoppedState;
     if (m_state != lastState)
-        Q_EMIT stateChanged(m_state);
+        stateChanged(m_state);
 }
 
 void AVFMediaEncoder::assetWriterError(QString err)
 {
-    Q_EMIT error(QMediaRecorder::FormatError, err);
+    updateError(QMediaRecorder::FormatError, err);
     if (m_state != QMediaRecorder::StoppedState)
         stopWriter();
 }
