@@ -20,18 +20,26 @@
 
 #if defined(QT_PLATFORM_UIKIT)
 #include <CoreVideo/CVBase.h>
+
+@interface DisplayLinkObserver : NSObject
+- (void)start;
+- (void)stop;
+- (void)displayLinkNotification:(CADisplayLink *)sender;
+@end
 #else
 #include <QuartzCore/CVDisplayLink.h>
 #endif
 
+#include <optional>
+
 QT_BEGIN_NAMESPACE
 
-class AVFDisplayLink : public QObject
+class AVFDisplayLink final : public QObject
 {
     Q_OBJECT
 public:
     explicit AVFDisplayLink(QObject *parent = nullptr);
-    virtual ~AVFDisplayLink();
+    ~AVFDisplayLink() override;
     bool isValid() const;
     bool isActive() const;
 
@@ -46,18 +54,17 @@ public:
     void displayLinkEvent(const CVTimeStamp *);
 
 protected:
-    virtual bool event(QEvent *) override;
+    bool event(QEvent *) override;
 
 private:
 #if defined(QT_PLATFORM_UIKIT)
-    void *m_displayLink;
+    DisplayLinkObserver *m_displayLink{};
 #else
-    CVDisplayLinkRef m_displayLink;
+    CVDisplayLinkRef m_displayLink{};
 #endif
     QMutex m_displayLinkMutex;
-    bool m_pendingDisplayLinkEvent;
-    bool m_isActive;
-    CVTimeStamp m_frameTimeStamp;
+    bool m_isActive{};
+    std::optional<CVTimeStamp> m_frameTimeStamp; // GUARDED_BY(m_displayLinkMutex)
 };
 
 QT_END_NAMESPACE
