@@ -3,12 +3,14 @@
 
 #include "qffmpeg_p.h"
 
-#include <qdebug.h>
-#include <qloggingcategory.h>
+#include <QtCore/qdebug.h>
+#include <QtCore/qloggingcategory.h>
+#include <QtCore/qscopeguard.h>
 
 extern "C" {
 #include <libavutil/pixdesc.h>
 #include <libavutil/samplefmt.h>
+#include <libavutil/error.h>
 
 #ifdef Q_OS_DARWIN
 #include <libavutil/hwcontext_videotoolbox.h>
@@ -17,20 +19,20 @@ extern "C" {
 
 QT_BEGIN_NAMESPACE
 
-static Q_LOGGING_CATEGORY(qLcFFmpegUtils, "qt.multimedia.ffmpeg.utils");
+Q_STATIC_LOGGING_CATEGORY(qLcFFmpegUtils, "qt.multimedia.ffmpeg.utils");
 
 namespace QFFmpeg {
 
-bool isAVFormatSupported(const AVCodec *codec, PixelOrSampleFormat format)
+bool isAVFormatSupported(const Codec &codec, PixelOrSampleFormat format)
 {
-    if (codec->type == AVMEDIA_TYPE_VIDEO) {
+    if (codec.type() == AVMEDIA_TYPE_VIDEO) {
         auto checkFormat = [format](AVPixelFormat f) { return f == format; };
-        return findAVPixelFormat(codec, checkFormat) != AV_PIX_FMT_NONE;
+        return findAVPixelFormat(codec, checkFormat).has_value();
     }
 
-    if (codec->type == AVMEDIA_TYPE_AUDIO) {
-        const auto sampleFormats = getCodecSampleFormats(codec);
-        return hasAVValue(sampleFormats, AVSampleFormat(format));
+    if (codec.type() == AVMEDIA_TYPE_AUDIO) {
+        const auto sampleFormats = codec.sampleFormats();
+        return hasValue(sampleFormats, AVSampleFormat(format));
     }
 
     return false;
@@ -42,16 +44,11 @@ bool isHwPixelFormat(AVPixelFormat format)
     return desc && (desc->flags & AV_PIX_FMT_FLAG_HWACCEL) != 0;
 }
 
-bool isAVCodecExperimental(const AVCodec *codec)
+void applyExperimentalCodecOptions(const Codec &codec, AVDictionary **opts)
 {
-    return (codec->capabilities & AV_CODEC_CAP_EXPERIMENTAL) != 0;
-}
-
-void applyExperimentalCodecOptions(const AVCodec *codec, AVDictionary** opts)
-{
-    if (isAVCodecExperimental(codec)) {
+    if (codec.isExperimental()) {
         qCWarning(qLcFFmpegUtils) << "Applying the option 'strict -2' for the experimental codec"
-                                  << codec->name << ". it's unlikely to work properly";
+                                  << codec.name() << ". it's unlikely to work properly";
         av_dict_set(opts, "strict", "-2", 0);
     }
 }
@@ -169,7 +166,13 @@ SwrContextUPtr createResampleContext(const AVAudioFormat &inputFormat,
 
 #endif
 
-    swr_init(resampler);
+    auto error = QFFmpeg::AVError{
+        swr_init(resampler),
+    };
+    if (error != QFFmpeg::AVError::Success) {
+        qCWarning(qLcFFmpegUtils) << "Failed to initialize audio resampler:" << error;
+        return nullptr;
+    }
     return SwrContextUPtr(resampler);
 }
 
@@ -306,7 +309,7 @@ AVHWDeviceContext* avFrameDeviceContext(const AVFrame* frame) {
 }
 
 SwsContextUPtr createSwsContext(const QSize &srcSize, AVPixelFormat srcPixFmt, const QSize &dstSize,
-                                AVPixelFormat dstPixFmt, int conversionType)
+                                AVPixelFormat dstPixFmt, SwsFlags conversionType)
 {
 
     SwsContext *result =
@@ -338,14 +341,6 @@ std::string cvFormatToString(uint32_t cvFormat)
 
 #endif
 
-#if QT_FFMPEG_HAS_AVCODEC_GET_SUPPORTED_CONFIG
-void logGetCodecConfigError(const AVCodec *codec, AVCodecConfig config, int error)
-{
-    qCWarning(qLcFFmpegUtils) << "Failed to retrieve config" << config << "for codec" << codec->name
-                              << "with error" << error << err2str(error);
-}
-#endif
-
 } // namespace QFFmpeg
 
 QDebug operator<<(QDebug dbg, const AVRational &value)
@@ -354,6 +349,42 @@ QDebug operator<<(QDebug dbg, const AVRational &value)
     return dbg;
 }
 
+QDebug operator<<(QDebug dbg, const AVDictionary &dict)
+{
+    char *buffer = 0;
+    auto freeBuffer = QScopeGuard([&] {
+        av_free(buffer);
+    });
+
+    int status = av_dict_get_string(&dict, &buffer, '=', ',');
+    if (status < 0 || !buffer)
+        return dbg << "Failed to print AVDictionary";
+
+    dbg << buffer;
+    return dbg;
+}
+
+QDebug operator<<(QDebug dbg, const QFFmpeg::AVDictionaryHolder &dict)
+{
+    const AVDictionary *rawDict = dict.opts;
+    if (rawDict)
+        return dbg << *rawDict;
+    else
+        return dbg << "Empty AVDictionaryHolder";
+}
+
+QDebug operator<<(QDebug dbg, QFFmpeg::AVError error)
+{
+    if (error == QFFmpeg::AVError::Success) {
+        dbg << "Success";
+        return dbg;
+    }
+
+    char errBuf[AV_ERROR_MAX_STRING_SIZE];
+    dbg << av_make_error_string(errBuf, AV_ERROR_MAX_STRING_SIZE, qToUnderlying(error));
+    return dbg;
+}
+
 #if QT_FFMPEG_HAS_AV_CHANNEL_LAYOUT
 QDebug operator<<(QDebug dbg, const AVChannelLayout &layout)
 {
