@@ -1,9 +1,7 @@
 // Copyright (C) 2021 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
-// TESTED_COMPONENT=src/multimedia
-
-#include <QtTest/QtTest>
+#include <QtTest/qtest.h>
 
 #include <qvideosink.h>
 #include <qvideoframe.h>
@@ -14,8 +12,20 @@
 #include <qmediarecorder.h>
 #include <qmediaplayer.h>
 
+#include <private/mediabackendutils_p.h>
+
 #include <vector>
 
+#ifdef Q_OS_ANDROID
+#include <QJniEnvironment>
+#include <QJniObject>
+Q_DECLARE_JNI_CLASS(Window, "android/view/Window")
+Q_DECLARE_JNI_CLASS(View, "android/view/View")
+Q_DECLARE_JNI_CLASS(WindowInsets, "android/view/WindowInsets")
+Q_DECLARE_JNI_CLASS(WindowInsetsType, "android/view/WindowInsets$Type")
+Q_DECLARE_JNI_CLASS(Insets, "android/graphics/Insets")
+#endif
+
 QT_USE_NAMESPACE
 
 /*
@@ -46,11 +56,23 @@ public:
         widget->setScreen(screen ? screen : QApplication::primaryScreen());
         widget->setWindowFlags(flags);
         widget->setGeometry(geometry);
+#ifdef Q_OS_ANDROID
+    // Android is not a Window System. When calling setGeometry() on the main widget, it will
+    // be displayed at the beginning of the screen. The x,y coordinates are ignored and lost.
+    // To make the test consistent on Android, let's remember the geometry and use
+    // it later in the paintEvent
+        widget->m_paintPosition = geometry;
+#endif
         widget->show();
 
         return widget;
     }
 
+#ifdef Q_OS_ANDROID
+    QRect m_paintPosition;
+    bool m_isBlinkingRectWhite = false;
+#endif
+
     void setColors(QColor firstColor, QColor secondColor)
     {
         m_firstColor = firstColor;
@@ -63,9 +85,18 @@ protected:
     {
         QPainter p(this);
         p.setPen(Qt::NoPen);
+        auto rect = this->rect();
+
+#ifdef Q_OS_ANDROID
+        // Add a blinking rectangle in the corner to force the Screen Grabber to work
+        m_isBlinkingRectWhite = !m_isBlinkingRectWhite;
+        p.setBrush(m_isBlinkingRectWhite ? Qt::white : Qt::black);
+        p.drawRect(0, 0, 10, 10);
+        // use remembered position
+        rect = m_paintPosition;
+#endif
 
         p.setBrush(m_firstColor);
-        auto rect = this->rect();
         p.drawRect(rect);
 
         if (m_firstColor != m_secondColor) {
@@ -82,8 +113,18 @@ private:
 
 class TestVideoSink : public QVideoSink
 {
+    Q_OBJECT
 public:
-    TestVideoSink() = default;
+    TestVideoSink()
+    {
+        connect(this, &QVideoSink::videoFrameChanged, this, &TestVideoSink::videoFrameChangedSync);
+
+#ifdef Q_OS_ANDROID
+        // Repaint to force the Screen Grabber to work
+        connect(this, &QVideoSink::videoFrameChanged, this, []() {
+            for (QWidget* widget : QApplication::topLevelWidgets()) { widget->update(); }});
+#endif
+    }
 
     void setStoreImagesEnabled(bool storeImages = true) {
         if (storeImages)
@@ -94,6 +135,15 @@ public:
 
     const std::vector<QImage> &images() const { return m_images; }
 
+    QVideoFrame waitForFrame()
+    {
+        QSignalSpy spy(this, &TestVideoSink::videoFrameChangedSync);
+        return spy.wait() ? spy.at(0).at(0).value<QVideoFrame>() : QVideoFrame{};
+    }
+
+signals:
+    void videoFrameChangedSync(QVideoFrame frame);
+
 private:
     void storeImage(const QVideoFrame &frame) {
         auto image = frame.toImage();
@@ -129,13 +179,6 @@ private slots:
 
 void tst_QScreenCaptureBackend::setActive_startsAndStopsCapture()
 {
-#ifdef Q_OS_ANDROID
-    // Should be removed after fixing QTBUG-112855
-    auto widget = QTestWidget::createAndShow(Qt::Window | Qt::FramelessWindowHint,
-                                             QRect{ 200, 100, 430, 351 });
-    QVERIFY(QTest::qWaitForWindowExposed(widget.get()));
-    QTest::qWait(100);
-#endif
     TestVideoSink sink;
     QScreenCapture sc;
 
@@ -214,7 +257,20 @@ void tst_QScreenCaptureBackend::capture(QTestWidget &widget, const QPoint &drawi
 
     sc.setActive(true);
 
-    QTest::qWait(300);
+    QVERIFY(sc.isActive());
+
+#ifdef Q_OS_LINUX
+    // In some cases, on Linux the window seems to be of a wrong color after appearance,
+    // the delay helps.
+    // TODO: remove the delay
+    QTest::qWait(2000);
+#endif
+    // Let's wait for the first frame to address a potential initialization delay.
+    // In practice, the delay varies between the platform and may randomly get increased.
+    {
+        const auto firstFrame = sink.waitForFrame();
+        QVERIFY(firstFrame.isValid());
+    }
 
     sink.setStoreImagesEnabled();
 
@@ -225,7 +281,7 @@ void tst_QScreenCaptureBackend::capture(QTestWidget &widget, const QPoint &drawi
             delay / static_cast<int>(1000 / std::min(widget.screen()->refreshRate(), 60.));
     const int framesCount = static_cast<int>(sink.images().size());
     QCOMPARE_LE(framesCount, expectedFramesCount + 2);
-    QCOMPARE_GE(framesCount, expectedFramesCount / 2);
+    QCOMPARE_GE(framesCount, 1);
 
     for (const auto &image : sink.images()) {
         auto pixelColor = [&drawingOffset, pixelRatio, &image](int x, int y) {
@@ -285,14 +341,62 @@ void tst_QScreenCaptureBackend::removeWhileCapture(
     QVERIFY2(framesSpy.empty(), "No frames expected after screen removal");
 }
 
+int getStatusBarHeight([[maybe_unused]] const qreal pixelRatio = 1)
+{
+#ifdef Q_OS_ANDROID
+
+    using namespace QtJniTypes;
+
+    static int statusBarHeight = -1;
+
+    if (statusBarHeight > -1)
+        return statusBarHeight;
+
+    auto activity = QNativeInterface::QAndroidApplication::context();
+    auto window = activity.callMethod<Window>("getWindow");
+    if (window.isValid()) {
+        auto decorView = window.callMethod<View>("getDecorView");
+        if (decorView.isValid()) {
+            auto rootInsets = decorView.callMethod<WindowInsets>("getRootWindowInsets");
+            if (rootInsets.isValid()) {
+                if (QNativeInterface::QAndroidApplication::sdkVersion() >= 30) {
+                    int windowInsetsType = WindowInsetsType::callStaticMethod<jint>("statusBars");
+                    auto insets = rootInsets.callMethod<Insets>(
+                        "getInsetsIgnoringVisibility", windowInsetsType);
+                    if (rootInsets.isValid())
+                        statusBarHeight = insets.getField<jint>("top");
+                } else {
+                    statusBarHeight = rootInsets.callMethod<jint>("getStableInsetTop");
+                }
+            }
+        }
+    }
+
+    if (statusBarHeight == -1) {
+        qWarning() << "Failed to get status bar height, falling back to zero.";
+        return 0;
+    }
+
+    if (pixelRatio != 0)
+        statusBarHeight /= pixelRatio;
+
+    return statusBarHeight;
+#else
+    return 0;
+#endif
+}
+
 void tst_QScreenCaptureBackend::initTestCase()
 {
 #ifdef Q_OS_ANDROID
-    QSKIP("grabWindow() no longer supported on Android adding child windows support: QTBUG-118849");
+    // QTBUG-132249:
+    // Security Popup can be automatically accepted with adb command:
+    // "adb shell appops set org.qtproject.example.tst_qscreencapturebackend PROJECT_MEDIA allow"
+    // Need to find a way to call it by androidtestrunner after installation and before running the test
+    QSKIP("Skip on Android; There is a security popup that need to be accepted");
 #endif
 #if defined(Q_OS_LINUX)
-    if (qEnvironmentVariable("QTEST_ENVIRONMENT").toLower() == "ci" &&
-        qEnvironmentVariable("XDG_SESSION_TYPE").toLower() != "x11")
+    if (isCI() && qEnvironmentVariable("XDG_SESSION_TYPE").toLower() != "x11")
         QSKIP("Skip on wayland; to be fixed");
 #endif
 
@@ -306,36 +410,30 @@ void tst_QScreenCaptureBackend::initTestCase()
 
 void tst_QScreenCaptureBackend::setScreen_selectsScreen_whenCalledWithWidgetsScreen()
 {
-    auto widget = QTestWidget::createAndShow(Qt::Window | Qt::FramelessWindowHint
-                                                     | Qt::WindowStaysOnTopHint
-#ifdef Q_OS_ANDROID
-                                                     | Qt::Popup
-#endif
-                                                     ,
-                                             QRect{ 200, 100, 430, 351 });
+    auto widget = QTestWidget::createAndShow(
+            Qt::Window | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint,
+            QRect{ 200, 100, 430, 351 });
     QVERIFY(QTest::qWaitForWindowExposed(widget.get()));
 
-    capture(*widget, { 200, 100 }, widget->screen()->size(),
+    const QPoint drawingOffset(200, 100 + getStatusBarHeight(widget->devicePixelRatio()));
+    capture(*widget, drawingOffset, widget->screen()->size(),
             [&widget](QScreenCapture &sc) { sc.setScreen(widget->screen()); });
 }
 
 void tst_QScreenCaptureBackend::constructor_selectsPrimaryScreenAsDefault()
 {
-    auto widget = QTestWidget::createAndShow(Qt::Window | Qt::FramelessWindowHint
-                                                     | Qt::WindowStaysOnTopHint
-#ifdef Q_OS_ANDROID
-                                                     | Qt::Popup
-#endif
-                                                     ,
-                                             QRect{ 200, 100, 430, 351 });
+    auto widget = QTestWidget::createAndShow(
+            Qt::Window | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint,
+            QRect{ 200, 100, 430, 351 });
     QVERIFY(QTest::qWaitForWindowExposed(widget.get()));
 
-    capture(*widget, { 200, 100 }, QApplication::primaryScreen()->size(), nullptr);
+    const QPoint drawingOffset(200, 100 + getStatusBarHeight(widget->devicePixelRatio()));
+    capture(*widget, drawingOffset, QApplication::primaryScreen()->size(), nullptr);
 }
 
 void tst_QScreenCaptureBackend::setScreen_selectsSecondaryScreen_whenCalledWithSecondaryScreen()
 {
-    const auto screens = QApplication::screens();
+    auto screens = QApplication::screens();
     if (screens.size() < 2)
         QSKIP("2 or more screens required");
 
@@ -350,26 +448,33 @@ void tst_QScreenCaptureBackend::setScreen_selectsSecondaryScreen_whenCalledWithS
             Qt::Window | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint,
             QRect{ 200, 100, 430, 351 }, screens.front(), QColor(0, 0, 0), QColor(0, 0, 0));
     QVERIFY(QTest::qWaitForWindowExposed(widgetOnPrimaryScreen.get()));
-    capture(*widgetOnSecondaryScreen, { 200, 100 }, screens.back()->size(),
+    const QPoint drawingOffset(200, 100 + getStatusBarHeight(widgetOnSecondaryScreen->devicePixelRatio()));
+    capture(*widgetOnSecondaryScreen, drawingOffset, screens.back()->size(),
             [&screens](QScreenCapture &sc) { sc.setScreen(screens.back()); });
 }
 
 void tst_QScreenCaptureBackend::capture_capturesToFile_whenConnectedToMediaRecorder()
 {
 #ifdef Q_OS_LINUX
-    if (qEnvironmentVariable("QTEST_ENVIRONMENT").toLower() == "ci")
+    if (isCI())
         QSKIP("QTBUG-116671: SKIP on linux CI to avoid crashes in ffmpeg. To be fixed.");
 #endif
 
     // Create widget with blue color
-    auto widget = QTestWidget::createAndShow(Qt::Window | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint,
-                                                 QRect{ 200, 100, 430, 351 });
+    auto widget = QTestWidget::createAndShow(
+            Qt::Window | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint,
+            QRect{ 200, 100, 430, 351 });
     widget->setColors(QColor(0, 0, 0xFF), QColor(0, 0, 0xFF));
 
     QScreenCapture sc;
     QSignalSpy errorsSpy(&sc, &QScreenCapture::errorOccurred);
     QMediaCaptureSession session;
     QMediaRecorder recorder;
+#ifdef Q_OS_ANDROID
+    // ADD dummy sink just for trigger repainting (for blinking rectangle)
+    TestVideoSink dummySink;
+    session.setVideoSink(&dummySink);
+#endif
     session.setScreenCapture(&sc);
     session.setRecorder(&recorder);
     auto screen = QApplication::primaryScreen();
@@ -380,13 +485,13 @@ void tst_QScreenCaptureBackend::capture_capturesToFile_whenConnectedToMediaRecor
 
     // Insert metadata
     QMediaMetaData metaData;
-    metaData.insert(QMediaMetaData::Author, QString::fromUtf8("Author"));
+    metaData.insert(QMediaMetaData::Author, QStringLiteral("Author"));
     metaData.insert(QMediaMetaData::Date, QDateTime::currentDateTime());
     recorder.setMetaData(metaData);
 
     sc.setActive(true);
 
-    QTest::qWait(200); // wait a bit for SC threading activating
+    QTest::qWait(1000); // wait a bit for SC threading activating
 
     {
         QSignalSpy recorderStateChanged(&recorder, &QMediaRecorder::recorderStateChanged);
@@ -397,9 +502,9 @@ void tst_QScreenCaptureBackend::capture_capturesToFile_whenConnectedToMediaRecor
         QCOMPARE(recorder.recorderState(), QMediaRecorder::RecordingState);
     }
 
-    QTest::qWait(300);
+    QTest::qWait(1000);
     widget->setColors(QColor(0, 0xFF, 0), QColor(0, 0xFF, 0)); // Change widget color
-    QTest::qWait(300);
+    QTest::qWait(1000);
 
     {
         QSignalSpy recorderStateChanged(&recorder, &QMediaRecorder::recorderStateChanged);
@@ -417,9 +522,11 @@ void tst_QScreenCaptureBackend::capture_capturesToFile_whenConnectedToMediaRecor
     TestVideoSink sink;
     QMediaPlayer player;
     player.setSource(fileName);
-    QCOMPARE_EQ(player.metaData().value(QMediaMetaData::Resolution).toSize(), QSize(videoResolution));
+    QTRY_COMPARE(player.mediaStatus(), QMediaPlayer::LoadedMedia);
+    QCOMPARE_EQ(player.metaData().value(QMediaMetaData::Resolution).toSize(),
+                QSize(videoResolution));
     QCOMPARE_GT(player.duration(), 350);
-    QCOMPARE_LT(player.duration(), 650);
+    QCOMPARE_LT(player.duration(), 3000);
 
     // Convert video frames to QImages
     player.setVideoSink(&sink);
@@ -429,9 +536,10 @@ void tst_QScreenCaptureBackend::capture_capturesToFile_whenConnectedToMediaRecor
     QTRY_COMPARE(player.mediaStatus(), QMediaPlayer::EndOfMedia);
     const size_t framesCount = sink.images().size();
 
-    // Find pixel point at center of widget
-    int x = 415 * videoResolution.width() / screenSize.width();
-    int y = 275 * videoResolution.height() / screenSize.height();
+    // Find pixel point at center of widget. Do not allow to get out of the frame size
+    int x = std::min(415 * videoResolution.width() / screenSize.width(), videoResolution.width() - 1);
+    int y = std::min(275 * videoResolution.height() / screenSize.height(), videoResolution.height() - 1);
+
     auto point = QPoint(x, y);
 
     // Verify color of first fourth of the video frames
@@ -439,7 +547,7 @@ void tst_QScreenCaptureBackend::capture_capturesToFile_whenConnectedToMediaRecor
         QImage image = sink.images().at(i);
         QVERIFY(!image.isNull());
         QRgb rgb = image.pixel(point);
-//        qDebug() << QString("RGB: %1, %2, %3").arg(qRed(rgb)).arg(qGreen(rgb)).arg(qBlue(rgb));
+//        qDebug() << QStringLiteral("RGB: %1, %2, %3").arg(qRed(rgb)).arg(qGreen(rgb)).arg(qBlue(rgb));
 
         // RGB values should be 0, 0, 255. Compensating for inaccurate video encoding.
         QVERIFY(qRed(rgb) <= 60);
@@ -452,7 +560,7 @@ void tst_QScreenCaptureBackend::capture_capturesToFile_whenConnectedToMediaRecor
         QImage image = sink.images().at(i);
         QVERIFY(!image.isNull());
         QRgb rgb = image.pixel(point);
-//        qDebug() << QString("RGB: %1, %2, %3").arg(qRed(rgb)).arg(qGreen(rgb)).arg(qBlue(rgb));
+//        qDebug() << QStringLiteral("RGB: %1, %2, %3").arg(qRed(rgb)).arg(qGreen(rgb)).arg(qBlue(rgb));
 
         // RGB values should be 0, 255, 0. Compensating for inaccurate video encoding.
         QVERIFY(qRed(rgb) <= 60);
